@@ -1,4 +1,4 @@
-import os, re, sys, subprocess
+import os, re, sys, subprocess, platform
 import tarfile
 from distutils import log, version
 from contextlib import closing
@@ -26,7 +26,7 @@ except:
 # use pre-built libraries on Windows
 
 def download_and_extract_windows_binaries(destdir):
-    url = "https://github.com/mhils/libxml2-win-binaries/releases"
+    url = "https://github.com/lxml/libxml2-win-binaries/releases"
     filenames = list(_list_dir_urllib(url))
 
     release_path = "/download/%s/" % find_max_version(
@@ -38,7 +38,13 @@ def download_and_extract_windows_binaries(destdir):
         if release_path in filename
     ]
 
-    arch = "win64" if sys.maxsize > 2**32 else "win32"
+    if platform.machine() == 'ARM64':
+        arch = "win-arm64"
+    elif sys.maxsize > 2**32:
+        arch = "win64"
+    else:
+        arch = "win32"
+
     if sys.version_info < (3, 5):
         arch = 'vs2008.' + arch
 
@@ -115,6 +121,7 @@ def get_prebuilt_libxml2xslt(download_dir, static_include_dirs, static_library_d
 ## Routines to download and build libxml2/xslt from sources:
 
 LIBXML2_LOCATION = 'http://xmlsoft.org/sources/'
+LIBXSLT_LOCATION = 'http://xmlsoft.org/sources/'
 LIBICONV_LOCATION = 'https://ftp.gnu.org/pub/gnu/libiconv/'
 ZLIB_LOCATION = 'https://zlib.net/'
 match_libfile_version = re.compile('^[^-]*-([.0-9-]+)[.].*').match
@@ -188,7 +195,7 @@ def parse_text_ftplist(s):
 
 def parse_html_filelist(s):
     re_href = re.compile(
-        r'<a\s+(?:[^>]*\s+)?href=["\']([^;?"\']+?)[;?"\']',
+        r'''<a[^>]*\shref=["']([^;?"']+?)[;?"']''',
         re.I|re.M)
     links = set(re_href.findall(s))
     for link in links:
@@ -208,7 +215,15 @@ def download_libxml2(dest_dir, version=None):
     #version_re = re.compile(r'LATEST_LIBXML2_IS_([0-9.]+[0-9](?:-[abrc0-9]+)?)')
     version_re = re.compile(r'libxml2-([0-9.]+[0-9]).tar.gz')
     filename = 'libxml2-%s.tar.gz'
-    return download_library(dest_dir, LIBXML2_LOCATION, 'libxml2',
+
+    if version == "2.9.12":
+        # Temporarily using the latest master (2.9.12+) until there is a release that supports lxml again.
+        from_location = "https://gitlab.gnome.org/GNOME/libxml2/-/archive/dea91c97debeac7c1aaf9c19f79029809e23a353/"
+        version = "dea91c97debeac7c1aaf9c19f79029809e23a353"
+    else:
+        from_location = LIBXML2_LOCATION
+
+    return download_library(dest_dir, from_location, 'libxml2',
                             version_re, filename, version=version)
 
 
@@ -217,7 +232,7 @@ def download_libxslt(dest_dir, version=None):
     #version_re = re.compile(r'LATEST_LIBXSLT_IS_([0-9.]+[0-9](?:-[abrc0-9]+)?)')
     version_re = re.compile(r'libxslt-([0-9.]+[0-9]).tar.gz')
     filename = 'libxslt-%s.tar.gz'
-    return download_library(dest_dir, LIBXML2_LOCATION, 'libxslt',
+    return download_library(dest_dir, LIBXSLT_LOCATION, 'libxslt',
                             version_re, filename, version=version)
 
 
@@ -371,8 +386,29 @@ def build_libxml2xslt(download_dir, build_dir,
     libxml2_dir  = unpack_tarball(download_libxml2(download_dir, libxml2_version), build_dir)
     libxslt_dir  = unpack_tarball(download_libxslt(download_dir, libxslt_version), build_dir)
     prefix = os.path.join(os.path.abspath(build_dir), 'libxml2')
+    lib_dir = os.path.join(prefix, 'lib')
     safe_mkdir(prefix)
 
+    lib_names = ['libxml2', 'libexslt', 'libxslt', 'iconv', 'libz']
+    existing_libs = {
+        lib: os.path.join(lib_dir, filename)
+        for lib in lib_names
+        for filename in os.listdir(lib_dir)
+        if lib in filename and filename.endswith('.a')
+    } if os.path.isdir(lib_dir) else {}
+
+    def has_current_lib(name, build_dir, _build_all_following=[False]):
+        if _build_all_following[0]:
+            return False  # a dependency was rebuilt => rebuilt this lib as well
+        lib_file = existing_libs.get(name)
+        found = lib_file and os.path.getmtime(lib_file) > os.path.getmtime(build_dir)
+        if found:
+            print("Found pre-built '%s'" % name)
+        else:
+            # also rebuild all following libs (which may depend on this one)
+            _build_all_following[0] = True
+        return found
+
     call_setup = {}
     if sys.platform == 'darwin':
         configure_darwin_env(call_setup)
@@ -388,10 +424,12 @@ def build_libxml2xslt(download_dir, build_dir,
         './configure',
         '--prefix=%s' % prefix,
     ]
-    cmmi(zlib_configure_cmd, zlib_dir, multicore, **call_setup)
+    if not has_current_lib("libz", zlib_dir):
+        cmmi(zlib_configure_cmd, zlib_dir, multicore, **call_setup)
 
     # build libiconv
-    cmmi(configure_cmd, libiconv_dir, multicore, **call_setup)
+    if not has_current_lib("iconv", libiconv_dir):
+        cmmi(configure_cmd, libiconv_dir, multicore, **call_setup)
 
     # build libxml2
     libxml2_configure_cmd = configure_cmd + [
@@ -411,7 +449,20 @@ def build_libxml2xslt(download_dir, build_dir,
             libxml2_configure_cmd.append('--enable-rebuild-docs=no')
     except Exception:
         pass # this isn't required, so ignore any errors
-    cmmi(libxml2_configure_cmd, libxml2_dir, multicore, **call_setup)
+    if not has_current_lib("libxml2", libxml2_dir):
+        if not os.path.exists(os.path.join(libxml2_dir, "configure")):
+            # Allow building from git sources by running autoconf etc.
+            libxml2_configure_cmd[0] = "./autogen.sh"
+        cmmi(libxml2_configure_cmd, libxml2_dir, multicore, **call_setup)
+
+    # Fix up libxslt configure script (needed up to and including 1.1.34)
+    # https://gitlab.gnome.org/GNOME/libxslt/-/commit/90c34c8bb90e095a8a8fe8b2ce368bd9ff1837cc
+    with open(os.path.join(libxslt_dir, "configure"), 'rb') as f:
+        config_script = f.read()
+    if b' --libs print ' in config_script:
+        config_script = config_script.replace(b' --libs print ', b' --libs ')
+        with open(os.path.join(libxslt_dir, "configure"), 'wb') as f:
+            f.write(config_script)
 
     # build libxslt
     libxslt_configure_cmd = configure_cmd + [
@@ -419,13 +470,13 @@ def build_libxml2xslt(download_dir, build_dir,
         '--with-libxml-prefix=%s' % prefix,
         '--without-crypto',
     ]
-    cmmi(libxslt_configure_cmd, libxslt_dir, multicore, **call_setup)
+    if not (has_current_lib("libxslt", libxslt_dir) and has_current_lib("libexslt", libxslt_dir)):
+        cmmi(libxslt_configure_cmd, libxslt_dir, multicore, **call_setup)
 
     # collect build setup for lxml
     xslt_config = os.path.join(prefix, 'bin', 'xslt-config')
     xml2_config = os.path.join(prefix, 'bin', 'xml2-config')
 
-    lib_dir = os.path.join(prefix, 'lib')
     static_include_dirs.extend([
             os.path.join(prefix, 'include'),
             os.path.join(prefix, 'include', 'libxml2'),
@@ -435,7 +486,7 @@ def build_libxml2xslt(download_dir, build_dir,
 
     listdir = os.listdir(lib_dir)
     static_binaries += [os.path.join(lib_dir, filename)
-        for lib in ['libxml2', 'libexslt', 'libxslt', 'iconv', 'libz']
+        for lib in lib_names
         for filename in listdir
         if lib in filename and filename.endswith('.a')]
 
